#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['fastgenomics']

package_data = \
{'': ['*'], 'fastgenomics': ['schemes/*', 'templates/*']}

install_requires = \
['jsonschema~=2.6', 'Jinja2~=2.9']

setup(name='fastgenomics',
      version='0.6.3',
      description='FASTGenomics python helper',
      author='FASTGenomics Team',
      author_email='contact@fastgenomics.org',
      url='https://github.com/fastgenomics/fastgenomics-py',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      python_requires='~=3.6',
     )
